/*******************
 * Interface builder
 *******************/
/*
 * Include declarations
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <hildon/hildon.h>
#include <hildon/hildon-gtk.h>
#include <gtk/gtk.h>
#include <gdk/gdkpixbuf.h>

#include <libraw/libraw.h>

#include "interface.h"
#include "callbacks.h"
#include "../config.h"

/*
 * Main application's window
 */
void create_app(app_data_t *myapp) {
	GtkWidget *openitem;
	GtkWidget *showimgdet;
	GtkWidget *hideimgdet;
	GtkWidget *aboutitem;
	GtkWidget *quititem;
	GtkWidget *previtem;
	GtkWidget *nextitem;
	GtkWidget *panarea;
    GtkWidget *eventbox;

    /* Create the hildon program and setup the title */
    myapp->app = HILDON_PROGRAM(hildon_program_get_instance());
    g_set_application_name("RAW Viewer");

    /* Create HildonWindow and set it to HildonProgram */
    myapp->mainwin = HILDON_WINDOW (hildon_window_new());
    hildon_gtk_window_set_portrait_flags(GTK_WINDOW (myapp->mainwin), HILDON_PORTRAIT_MODE_SUPPORT);
    hildon_program_add_window(myapp->app, myapp->mainwin);

    /* Create the main menu */
    myapp->appmenu = HILDON_APP_MENU(hildon_app_menu_new());

    previtem = gtk_button_new_with_label("Previous image");
    gtk_widget_set_sensitive(previtem,FALSE);
    g_signal_connect_after(G_OBJECT (previtem),"clicked",G_CALLBACK(load_prev_image),myapp);
    hildon_app_menu_insert(myapp->appmenu,GTK_BUTTON(previtem),0);

    nextitem = gtk_button_new_with_label("Next image");
    gtk_widget_set_sensitive(nextitem,FALSE);
    g_signal_connect_after(G_OBJECT (nextitem),"clicked",G_CALLBACK(load_next_image),myapp);
    hildon_app_menu_insert(myapp->appmenu,GTK_BUTTON(nextitem),1);

    openitem = gtk_button_new_with_label("Open image");
    g_signal_connect_after(G_OBJECT (openitem),"clicked",G_CALLBACK(open_image_file),myapp);
    hildon_app_menu_insert(myapp->appmenu,GTK_BUTTON(openitem),2);

    aboutitem = gtk_button_new_with_label("About");
    g_signal_connect_after(G_OBJECT (aboutitem),"clicked",G_CALLBACK(create_about),myapp);
    hildon_app_menu_insert(myapp->appmenu,GTK_BUTTON(aboutitem),3);

    quititem = gtk_button_new_with_label("Quit");
    g_signal_connect_after(G_OBJECT (quititem),"clicked",G_CALLBACK(gtk_main_quit),myapp);
    hildon_app_menu_insert(myapp->appmenu,GTK_BUTTON(quititem),4);

    showimgdet = gtk_radio_button_new_with_label(NULL,"Show details");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (showimgdet),FALSE);
    gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON (showimgdet),FALSE);
    hideimgdet = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (showimgdet),"Hide details");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (hideimgdet),TRUE);
    gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON (hideimgdet),FALSE);
    g_signal_connect_after(G_OBJECT (showimgdet),"clicked",G_CALLBACK(show_image_details),myapp);
    g_signal_connect_after(G_OBJECT (hideimgdet),"clicked",G_CALLBACK(hide_image_details),myapp);
    hildon_app_menu_add_filter(myapp->appmenu,GTK_BUTTON (showimgdet));
    hildon_app_menu_add_filter(myapp->appmenu,GTK_BUTTON (hideimgdet));
    gtk_widget_show_all(GTK_WIDGET (myapp->appmenu));

    hildon_window_set_app_menu(HILDON_WINDOW (myapp->mainwin),myapp->appmenu);

	/* Add a pannable area with the image container inside */
    panarea = GTK_WIDGET (hildon_pannable_area_new());
    gtk_widget_show(GTK_WIDGET (panarea));
    gtk_container_add(GTK_CONTAINER (myapp->mainwin),panarea);
    eventbox = gtk_event_box_new();
    gtk_widget_show(eventbox);
    gtk_widget_set_events(eventbox,GDK_BUTTON_PRESS | GDK_BUTTON_RELEASE | GDK_KEY_PRESS);
    myapp->image = GTK_WIDGET (gtk_image_new());
    gtk_widget_show(GTK_WIDGET (myapp->image));
    gtk_container_add(GTK_CONTAINER (eventbox),myapp->image);
    hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA (panarea),eventbox);
    hildon_pannable_area_set_size_request_policy(HILDON_PANNABLE_AREA (panarea),HILDON_SIZE_REQUEST_CHILDREN);

    /* Show everything */
    gtk_widget_show_all(GTK_WIDGET (myapp->mainwin));

    /*
     * Connect some signals
     */
    g_signal_connect(G_OBJECT (myapp->mainwin),"delete_event",
    		G_CALLBACK (gtk_main_quit),NULL);
    g_signal_connect(G_OBJECT (myapp->mainwin),"configure_event",
    		G_CALLBACK (detect_main_window_changes),myapp);
    g_signal_connect(G_OBJECT (eventbox),"button_press_event",
    		G_CALLBACK (detect_button_press_on_image),myapp);
    g_signal_connect(G_OBJECT (eventbox),"button_release_event",
        	G_CALLBACK (detect_button_press_on_image),myapp);
    g_signal_connect(G_OBJECT(myapp->mainwin),"key_press_event",
			G_CALLBACK (detect_hardware_key_press),myapp);

}

/*
 * Create the About dialog
 */
void create_about(GtkWidget *callerobj,app_data_t *myapp) {
	GtkWidget *abdialog;
	GtkWidget *appicon;
	GtkWidget *copyright;
	GtkWidget *panarea;
	GtkWidget *dispmessage;
	gchar *message;
	const char **cameralist;
	gint i;

	abdialog = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW (abdialog),g_strconcat("About ",PACKAGE," ",VERSION,NULL));
	gtk_window_set_default_size(GTK_WINDOW (abdialog),-1,400);
	appicon = GTK_WIDGET (gtk_image_new_from_file(g_strconcat(HILDONPIXMAPDIR,"/",PACKAGE,".png",NULL)));
	gtk_box_pack_start(GTK_BOX (GTK_DIALOG (abdialog)->vbox),appicon,FALSE,FALSE,10);
	copyright = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL (copyright),"<span size=\"small\" style=\"italic\">(C) 2009 Luca Donaggio (donaggio@gmail.com)</span>");
	gtk_box_pack_start(GTK_BOX (GTK_DIALOG (abdialog)->vbox),copyright,FALSE,FALSE,10);
	panarea = GTK_WIDGET (hildon_pannable_area_new());
	dispmessage = gtk_label_new(NULL);
	gtk_label_set_line_wrap(GTK_LABEL (dispmessage),TRUE);
	message = g_strconcat("<b>",PACKAGE,"</b> is an image viewer for the Maemo platform supporting RAW image file formats as those produced by modern DSLRs.\n\n"
						  "For an exahustive list of supported cameras please see below.\n\n",
						  "<b>",PACKAGE,"</b> is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.\n\n",
						  "<b>",PACKAGE,"</b> is based on LibRaw ",libraw_version()," which in turn is (C) 2008-2009 LibRaw LLC (info@libraw.org)\n\n",
						  "Supported cameras:\n\n<small>",NULL);
	cameralist = libraw_cameraList();
	for(i = 0;i < libraw_cameraCount();i++) message = g_strconcat(message,"\t",cameralist[i],"\n",NULL);
	message = g_strconcat(message,"</small>",NULL);
	gtk_label_set_markup(GTK_LABEL (dispmessage),message);
	hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA (panarea),dispmessage);
	gtk_box_pack_end_defaults(GTK_BOX (GTK_DIALOG (abdialog)->vbox),panarea);
	gtk_widget_show_all(abdialog);
	gtk_dialog_run(GTK_DIALOG (abdialog));
	gtk_widget_destroy(abdialog);
}
